#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2013 - Ronan Delacroix
File Utils
:author: Ronan Delacroix
"""

import os
import sys
from . import sequential

ZIP_EXTENSIONS = ['bz2', 'gz', 'xz', 'bz2', 'rar', 'gz', 'tar', 'tbz2', 'tgz', 'zip', 'Z', '7z', 'xz', 'ace']


def full_file_list(scan_path):
    """
    Returns a list of all files in a folder and its subfolders (only files).
    """
    file_list = []
    path = os.path.abspath(scan_path)
    for root, dirs, files in os.walk(path):
        if len(files) != 0 and not '.svn' in root and not '.git' in root:
            for f in files:
                file_list.append(os.path.join(root, f))
    return file_list


def list_files(scan_path, contains=None):
    """
    Returns a list of all files in a folder, without subfolders (only files).
    """
    file_list = []
    path = os.path.abspath(scan_path)
    for f in sorted(os.listdir(path)):
        if contains and f.find(contains) < 0:
            continue
        filepath = os.path.join(path, f)
        if os.path.isfile(filepath):
            file_list.append(filepath)
    return file_list


def full_file_list_with_sequence(scan_path):
    """
    Returns a list of all files in a folder and its subfolders (only files).
    """
    
    file_list = []
    path = os.path.abspath(scan_path)
    
    for root, dirs, files in os.walk(path):
        if len(files) != 0 and not '.svn' in root and not '.git' in root:
            try:
                sc = sequential.SequentialFolder(str(root))
                if sc.sequence:
                    file_list.append(sc)
                    continue
            except Exception as e:
                pass
                
            for f in files:
                file_list.append(os.path.join(root, f))
    return file_list


def readlinkabs(l):
    """
    Return an absolute path for the destination 
    of a symlink
    """
    assert (os.path.islink(l))
    p = os.readlink(l)
    if os.path.isabs(p):
        return os.path.abspath(p)
    return os.path.abspath(os.path.join(os.path.dirname(l), p))


def unzip(filepath, output_path):
    """
    Unzip an archive file
    """
    filename = os.path.split(filepath)[1]
    (name, extension) = os.path.splitext(filename)
    extension = extension[1:].lower()
    extension2 = os.path.splitext(name)[1][1:].lower()

    if extension not in ZIP_EXTENSIONS:
        raise Exception("Impossible to extract archive file %s" % filepath)

    extract_command = "unzip"
    output_args = "-d"
    if extension == 'bz2' and extension2 == 'tar':
        extract_command = "tar -xjf"
        output_args = "-C"
    elif extension == 'gz' and extension2 == 'tar':
        extract_command = "tar -xzf"
        output_args = "-C"
    elif extension == 'xz' and extension2 == 'tar':
        extract_command = "tar -xJf"
        output_args = "-C"
    elif extension == 'bz2':
        extract_command = "bunzip2 -dc "
        output_args = ">"
        output_path = os.path.join(output_path, name)
    elif extension == 'rar':
        extract_command = "unrar x"
        output_args = ""
    elif extension == 'gz':
        extract_command = "gunzip"
        output_args = ""
    elif extension == 'tar':
        extract_command = "tar -xf"
        output_args = "-C"
    elif extension == 'tbz2':
        extract_command = "tar -xjf"
        output_args = "-C"
    elif extension == 'tgz':
        extract_command = "tar -xzf"
        output_args = "-C"
    elif extension == 'zip':
        extract_command = "unzip"
        output_args = "-d"
    elif extension == 'Z':
        extract_command = "uncompress"
        output_args = ""
    elif extension == '7z':
        extract_command = "7z x"
        output_args = ""
    elif extension == 'xz':
        extract_command = "unxz"
        output_args = ""
    elif extension == 'ace':
        extract_command = "unace"
        output_args = ""
    elif extension == 'iso':
        extract_command = "7z x"
        output_args = ""
    elif extension == 'arj':
        extract_command = "7z x"
        output_args = ""

    command = """%(extract_command)s "%(filepath)s" %(output_args)s "%(output_folder)s" """
    params = {
        'extract_command': extract_command,
        'filepath': filepath,
        'output_folder': output_path,
        'output_args': output_args,
    }
    result = os.system(command % params)
    return result


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2013 - Ronan Delacroix
Process Utils
:author: Ronan Delacroix
"""

import os
import sys
import subprocess
import signal
import uuid as UUID
from datetime import datetime
import time
import tempfile
import re
import shutil

import logging
from . import text as text_utils


def restart_program():
    """
    Restarts the current program.
    Note: this function does not return. Any cleanup action (like
    saving data) must be done before calling this function.
    """
    logging.debug("Restarting program...")
    python = sys.executable
    os.execl(python, python, * sys.argv)


def synchronized(lock):
    """
    Synchronization decorator; provide thread-safe locking on a function
    http://code.activestate.com/recipes/465057/
    """
    def wrap(f):
        def synchronize(*args, **kw):
            lock.acquire()
            try:
                return f(*args, **kw)
            finally:
                lock.release()
        return synchronize
    return wrap


def synchronized_limit(lock):
    """
    Synchronization decorator; provide thread-safe locking on a function
    http://code.activestate.com/recipes/465057/
    """
    def wrap(f):
        def synchronize(*args, **kw):
            if lock[1] < 10:
                lock[1] += 1
                lock[0].acquire()
                try:
                    return f(*args, **kw)
                finally:
                    lock[1] -= 1
                    lock[0].release()
            else:
                raise Exception('Too busy')
        return synchronize
    return wrap


def execute(command, return_output=True, log_file=None, log_settings=None, error_logfile=None, timeout=None, line_function=None, poll_timing = 0.01, logger=None, working_folder=None, env=None):
    """
        Execute a program and logs standard output into a file.

        :param return_output:      returns the STDOUT value if True or returns the return code
        :param logfile:            path where log file should be written ( displayed on STDOUT if not set)
        :param error_logfile:      path where error log file should be written ( displayed on STDERR if not set)
        :param timeout:            if set, it will kill the subprocess created when "timeout" seconds is reached. It will then raise an Exception.
        :param line_function:      set it to a "function pointer" for the function to be called each time a new line is written (line passed as a parameter).
        :param poll_timing:        wait time between timeout checks and std output check.

        :returns:   Standard output of the command or if return_output=False, it will give the "return code" of the command
    """

    tmp_log = False
    if log_settings:
        log_folder = log_settings.get('LOG_FOLDER')
    else:
        tmp_log = True
        log_folder = tempfile.mkdtemp()

    if not log_file:
        log_file = os.path.join(log_folder, "commands", "execute-command-logfile-%s.log" % UUID.uuid4())
        try:
            if not os.path.isdir(os.path.join(log_folder, "commands")):
                os.makedirs(os.path.join(log_folder, "commands"))
        except:
            pass
    if not logger:
        logger = logging.getLogger('command_execute')

    logfile_writer = open(log_file, 'a')
    header = "%s - Executing command (timeout=%s) :\n\t%s\n\n\n" % (datetime.now().isoformat(), timeout, command)
    logfile_writer.write(header)
    logfile_writer.flush()

    logfile_reader = open(log_file, 'rb')
    logfile_reader.seek(0, os.SEEK_END)
    logfile_start_position = logfile_reader.tell()

    if error_logfile:
        err_logfile_writer = open(error_logfile, 'a')
    else:
        err_logfile_writer = logfile_writer

    start = datetime.now()
    timeout_string = ""
    if timeout:
        timeout_string = "(timeout=%s)" % timeout
    logger.info(u"Executing command %s :\n\t\t%s" % (timeout_string, command) )

    # We use "exec <command>" as Popen launches a shell, that runs the command.
    # It will transform the child process "sh" into the "command exectable" because of the "exec".
    # Said more accuratly, it won't fork to create launch the command in a sub sub process.
    # Therefore, when you kill the child process, you kill the "command" process and not the unecessary "sh" parent process.
    if sys.platform != 'win32':
        command = u"exec %s" % text_utils.uni(command)
    process = subprocess.Popen(command, stdout=logfile_writer, stderr=err_logfile_writer, bufsize=1, shell=True, cwd=working_folder, env=env)

    while process.poll() == None:
        # In order to avoid unecessary cpu usage, we wait for "poll_timing" seconds ( default: 0.1 sec )
        time.sleep(poll_timing)

        # Timeout check
        if timeout != None:
            now = datetime.now()
            if (now - start).seconds> timeout:
                #process.terminate() ??
                os.kill(process.pid, signal.SIGKILL)
                os.waitpid(-1, os.WNOHANG)
                raise Exception("Command execution timed out (took more than %s seconds...)" % timeout)

        # Line function call:
        #   => if line_function is defined, we call it on each new line of the file.
        if line_function:
            o = text_utils.uni(logfile_reader.readline()).rstrip()
            while o != '':
                line_function(o)
                o = text_utils.uni(logfile_reader.readline()).rstrip()

    if not return_output:
        # Return result code and ensure we have waited for the end of sub process
        return process.wait()

    logfile_reader.seek(logfile_start_position, os.SEEK_SET) #back to the beginning of the file

    res = text_utils.uni(logfile_reader.read())

    try:
        logfile_reader.close()
        logfile_writer.close()
        err_logfile_writer.close()

        if tmp_log:
            shutil.rmtree(log_folder, ignore_errors=True)
    except:
        pass

    return res


def daemonize(umask=0, work_dir="/", max_fd=1024, redirect="/dev/null"):
    """
    When this function is called, the process is daemonized (by forking + killing its parent).
    It becomes a background task.
    It is useful to release the console.
    """
    if not redirect:
        redirect = "/dev/null"

    if hasattr(os, "devnull"):
        redirect = os.devnull

    try:
        pid = os.fork()
    except OSError as e:
        raise Exception("%s [%d]" % (e.strerror, e.errno))

    # first child
    if pid == 0:
        os.setsid()

        try:
            # Fork a second child.
            pid = os.fork()
        except OSError as e:
            raise Exception("%s [%d]" % (e.strerror, e.errno))

        # The second child.
        if pid == 0:
            os.chdir(work_dir)
            os.umask(umask)
        else:
            # exit first child
            os._exit(0)
    else:
        # Exit parent
        os._exit(0)

    #killing inherited file descriptors
    import resource
    maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
    if maxfd == resource.RLIM_INFINITY:
        maxfd = max_fd

    # close all file descriptors.
    for fd in range(0, maxfd):
        try:
            os.close(fd)
        except OSError:
            # ignored
            pass

    os.open(redirect, os.O_RDWR) # standard input

    # Duplicate standard
    os.dup2(0, 1)			# standard output (1)
    os.dup2(0, 2)			# standard error (2)

    return os.getpid()


def is_running(process):
    s = subprocess.Popen(["ps", "axw"], stdout=subprocess.PIPE)
    for x in s.stdout:
        if re.search(process, x):
            return True
    return False


if __name__ == "__main__":
    import log
    log.configure_logging('process_test')

    COMMAND = """ifconfig {}""".format("-a -v")

    def show_address(line):
        if "inet " in line:
            addr = line.strip().split(' ')[1]
            print("### ADDRESS FOUND : {} ###".format(addr))

    result = execute(COMMAND, line_function=show_address)
    #print(result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2013 - Ronan Delacroix
Text Utils
:author: Ronan Delacroix
"""
import json
import datetime

import os
import lxml.etree as etree
import re
import smtplib
import unicodedata
import six
import uuid as UUID
import base64

from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email.mime.text import MIMEText
from email.utils import COMMASPACE, formatdate
from email import encoders

if six.PY3:
    import html
else:
    import cgi as html


def normalize_text(text):
    return unicodedata.normalize('NFKD', text).encode('ascii', 'ignore').decode('utf-8')


def slugify(text, delim='-'):
    """Generates an slightly worse ASCII-only slug."""
    punctuation_re = re.compile(r'[\t !"#$%&\'()*\-/<=>?@\[\\\]^_`{|},.:]+')

    result = []
    for word in punctuation_re.split(text.lower()):
        word = normalize_text(word)
        if word:
            result.append(word)
    return delim.join(result)


def slugify_bytes(b):
    return base64.urlsafe_b64encode(b).decode('utf-8').strip('=')


def uuid_to_slug(uuid):
    if isinstance(uuid, str):
        b = UUID.UUID(uuid)
    elif isinstance(uuid, UUID.UUID):
        b = uuid.bytes
    elif isinstance(uuid, bytes):
        b = uuid
    else:
        b = bytes(uuid)
    return slugify_bytes(b)


def random_slug():
    return uuid_to_slug(UUID.uuid4().bytes)


def random_short_slug():
    return uuid_to_slug(UUID.uuid4().bytes[0:8])


def javascript_escape(s, quote_double_quotes=True):
    """
    Escape characters for javascript strings.
    """
    ustring_re = re.compile(u"([\u0080-\uffff])")

    def fix(match):
        return r"\u%04x" % ord(match.group(1))

    if type(s) == str:
        s = s.decode('utf-8')
    elif type(s) != six.text_type:
        raise TypeError(s)
    s = s.replace('\\', '\\\\')
    s = s.replace('\r', '\\r')
    s = s.replace('\n', '\\n')
    s = s.replace('\t', '\\t')
    s = s.replace("'", "\\'")
    if quote_double_quotes:
        s = s.replace('"', '&quot;')
    return str(ustring_re.sub(fix, s))


def send_mail(send_from, send_to, subject, text, server, files=None):
    """
    Send an email with attachments.
    :param send_from: from email adress
    :param send_to: to email adress
    :param subject: email subject
    :param text: text of the email in html
    :param server: SMTP server
    :param files: files to attach
    :return: None
    """
    if not files:
        files = []

    assert type(send_to) == list
    assert type(files) == list

    msg = MIMEMultipart()
    msg['From'] = send_from
    msg['To'] = COMMASPACE.join(send_to)
    msg['Date'] = formatdate(localtime=True)
    msg['Subject'] = subject

    msg.attach(MIMEText(text, 'html'))

    for f in files:
        part = MIMEBase('application', "octet-stream")
        fp = open(f, "rb")
        file_content = fp.read()
        part.set_payload(file_content)
        encoders.encode_base64(part)
        part.add_header('Content-Disposition', 'attachment; filename="%s"' % os.path.basename(f))
        msg.attach(part)

    smtp = smtplib.SMTP(server)
    smtp.sendmail(send_from, send_to, msg.as_string())
    smtp.close()
    return


def hms_to_seconds(time_string):
    """
    Converts string 'hh:mm:ss.ssssss' as a float
    """
    s = time_string.split(':')
    hours = int(s[0])
    minutes = int(s[1])
    secs = float(s[2])
    return hours * 3600 + minutes * 60 + secs

    
def seconds_to_hms_verbose(t):
    """
    Converts seconds float to 'H hours 8 minutes, 30 seconds' format
    """
    hours = int((t / 3600))
    mins = int((t / 60) % 60)
    secs = int(t % 60)
    return ' '.join([
        (hours + ' hour'+('s' if hours > 1 else '')) if hours > 0 else '',
        (mins + ' minute'+('s' if mins > 1 else '')) if mins > 0 else '',
        (secs + ' second'+('s' if secs > 1 else '')) if secs > 0 else ''
    ])


def seconds_to_hms(seconds):
    """
    Converts seconds float to 'hh:mm:ss.ssssss' format.
    """
    hours = int(seconds / 3600.0)
    minutes = int((seconds / 60.0) % 60.0)
    secs = float(seconds % 60.0)
    return "{0:02d}:{1:02d}:{2:02.6f}".format(hours, minutes, secs)


def str_to_bool(v):
    return str(v).lower() in ("yes", "on", "true", "y", "t", "1")


datetime_handler = lambda obj: obj.isoformat(sep=' ') if isinstance(obj, datetime.datetime) else None

render_xml = lambda _dict: dict_to_xml_string("xml", _dict)
render_json = lambda _dict: json.dumps(_dict, sort_keys=False, indent=4, default=datetime_handler)
render_html = lambda _dict: dict_to_html(_dict)
render_txt = lambda _dict: dict_to_plaintext(_dict)

mime_rendering_dict = {
    'text/html': render_html,
    'application/html': render_html,
    'application/xml': render_xml,
    'application/json': render_json,
    'text/plain': render_txt
}


def render_dict_from_mimetype(d, mimetype):
    return mime_rendering_dict.get(mimetype, render_json)(d)


def pretty_render(data, format='text', indent=0):
    """
    Render a dict based on a format
    """
    if format == 'json':
        return render_json(data)
    elif format == 'html':
        return render_html(data)
    elif format == 'xml':
        return render_xml(data)
    else:
        return dict_to_plaintext(data, indent=indent)


# DICT TO XML FUNCTION
def _dict_to_xml_recurse(parent, dictitem):
    if isinstance(dictitem, list):
        dictitem = {'item' : dictitem}
    if isinstance(dictitem, dict):
        for (tag, child) in dictitem.items():
            if str(tag) == '_text':
                parent.text = str(child)
            elif type(child) is type([]):
                # iterate through the array and convert
                for listchild in child:
                    elem = etree.Element(tag)
                    parent.append(elem)
                    _dict_to_xml_recurse(elem, listchild)
            elif len(tag)==36 and tag[8]=='-' and tag[13]=='-': #if uuid is name of the element we try to cook up something nice to display in xml
                uuid = tag
                tag = parent.tag.replace('_list', '').replace('_dict', '')
                elem = etree.Element(tag, uuid=uuid)
                parent.append(elem)
                _dict_to_xml_recurse(elem, child)
            else:
                try:
                    elem = etree.Element(tag)
                except ValueError:
                    elem = etree.Element("element", unrecognized=tag)
                parent.append(elem)
                _dict_to_xml_recurse(elem, child)
    else:
        parent.text = str(dictitem)


def dict_to_xml(xml_dict):
    """
    Converts a dictionary to an XML ElementTree Element
    """
    root_tag = list(xml_dict.keys())[0]
    root = etree.Element(root_tag)
    _dict_to_xml_recurse(root, xml_dict[root_tag])
    return root


def dict_to_xml_string(root_name, _dict):
    _dict = {root_name: _dict}
    xml_root = dict_to_xml(_dict)
    return etree.tostring(xml_root, pretty_print=True, encoding="UTF-8", xml_declaration=True)


# DICT TO TEXT FUNCTION
def dict_to_plaintext(_dict, indent=0, result=''):
    if isinstance(_dict, list):
        i = 0
        if not _dict:
            result += '\t' * indent + "<empty>\n"
        for value in _dict:
            i += 1
            if isinstance(value, dict) :
                result += '\t' * indent + "["+str(i)+"]={DICT}\n" + dict_to_plaintext(value, indent+1)
            elif isinstance(value, list) :
                result += '\t' * indent + "["+str(i)+"]=<LIST>\n" + dict_to_plaintext(value, indent+1) + "\n"
            else:
                result += '\t' * indent + "["+str(i)+"]=\"" + str(value) + "\"\n"
        return result
    elif isinstance(_dict, dict):
        for key, value in _dict.items():
            if isinstance(value, dict):
                result += '\t' * indent + "{" + str(key) + "}\n" + dict_to_plaintext(value, indent+1)
            elif isinstance(value, list):
                result += '\t' * indent + "<" + str(key) + '>\n' + dict_to_plaintext(value, indent+1)
            else:
                if "\n" in str(value):
                    value = ' '.join([line.strip() for line in str(value).replace("\"", "'").split("\n")])
                result += '\t' * indent + str(key) + '=' + "\"" + str(value) + "\"\n"
        return result
    else:
        return "\"" + str(_dict) + "\""


# DICT TO HTML FUNCTION
def _dict_to_html_recurse(_dict, indent=0, result=''):
    if isinstance(_dict, list):
        i = 0
        result += '    ' * indent + "<ul>\n"
        for value in _dict:
            i += 1
            if isinstance(value, dict):
                result += '    ' * (indent+1) + "<li class='row"+str(i % 2)+"'>\n" + _dict_to_html_recurse(value, indent + 2) + '    ' * (indent + 1) + "</li>\n"
            elif isinstance(value, list):
                result += '    ' * (indent+1) + "<li class='row"+str(i % 2)+"'>\n" + _dict_to_html_recurse(value, indent + 2) + '    ' * (indent + 1) + "</li>\n"
            else:
                result += '    ' * (indent+1) + "<li class='row"+str(i % 2)+"'><pre>" + html.escape(str(value)) + "</pre></li>\n"
        result += '    ' * indent + "</ul>\n"
        return result
    elif isinstance(_dict, dict):
        result += '    ' * indent + "<table>\n"
        i=0
        for key, value in _dict.items():
            i+=1
            if isinstance(value, dict) or isinstance(value, list):
                result += '    ' * (indent + 1) + "<tr class='row"+str(i % 2)+"'>\n"
                result += '    ' * (indent + 2) + "<td>" + str(key) + "</td>\n"
                result += '    ' * (indent + 2) + "<td>\n" + _dict_to_html_recurse(value, indent+3)
                result += '    ' * (indent + 2) + "</td>\n"
                result += '    ' * (indent + 1) + "</tr>\n"
            else:
                value = html.escape(str(value))
                result += '    ' * (indent + 1) + "<tr class='row"+str(i % 2)+"'><td>" + str(key) + "</td><td><pre>" + str(value) + "</pre></td></tr>\n"
        result += '    ' * indent + "</table>\n"
        return result
    else:
        return "<pre>" + html.escape(str(_dict)) + "</pre>"


def dict_to_html(_dict, title="Result"):
    return """
<html>
    <head>
        <style>
            body { font-family: monospace; }
            table { display : inline-block; border-spacing: 0px; border-collapse: collapse; }
            td { border : 1px solid grey; padding:3px 10px; }
            li { border : 1px solid grey; padding:0px 10px 0px 10px; margin: 0px 0px 0px 5px; list-style-type : circle; }
            ul { display : inline-block; padding:0px 0px 0px 10px; margin:0px;}
            pre { margin:0 ; }
            .row0 { background-color:#EAEAFF; }
            .row1 { background-color:#FFFFFF; }
        </style>
        <title>"""+title+"""</title>
    </head>
    <body>
""" + _dict_to_html_recurse(_dict, 2) + "    </body>\n</html>"


def test_page(title="Result"):
    result = "<table>"
    docu = {}
    i=0
    for func_name, doc in docu.items():
        result += "<tr class='row" + str(i) + "'><td>" + doc['friendly_name'] + "</td>"
        if 'parameters' in doc:
            result += "<td><form action='" + func_name + "' method='"+doc['method_type']+"' enctype='multipart/form-data'>"
            result += "<table width='100%'>"
            if 'required' in doc['parameters']:
                result += "<tr><th colspan='2'>Required</th></tr>"
                for param in doc['parameters']['required']:
                    if param == 'asset_file':
                        result += "<tr><td>" + str(param) + "</td><td><input type='file' name='" + str(param) + "' value=''/></td><tr/>"
                    else:
                        result += "<tr><td>" + str(param) + "</td><td><input type='text' name='" + str(param) + "' value=''/></td><tr/>"

            if 'optionnal' in doc['parameters']:
                result += "<tr><th colspan='2'>Optionnal</th></tr>"
                for param, value in doc['parameters']['optionnal'].items():
                    if value==None:
                        value=''
                    result += "<tr><td>" + str(param) + "</td><td><input type='text' name='" + str(param) + "' value='" + str(value) + "'/></td><tr/>"
            result += "<tr><th colspan='2'><input type='submit'/></th></tr>"
            result += "</table>"
            result += "</form></td>"
        else:
            result += "<td><a href='" + func_name + "'>" + func_name + "</a></td>"
        result += "</tr>"
        i += 1
        i = i%2

    result += "</table>"
    return """
<html>
    <head>
        <style>
            body {font-family: monospace;}
            table {display : inline-block; border-spacing: 0px; border-collapse: collapse;}
            td {border: 1px solid grey; padding: 3px 10px;}
            li {border: 1px solid grey; padding: 0px 10px 0px 10px; margin: 0px 0px 0px 5px; list-style-type: circle;}
            ul {display: inline-block; padding: 0px 0px 0px 10px; margin:0px;}
            pre {margin: 0;}
            .row0 {background-color:#EAEAFF;}
            .row1 {background-color:#FFFFFF;}
        </style>
        <title>"""+title+"""</title>
    </head>
    <body>
""" + result + """
    </body>
</html>"""


def uni(text):
    """
    Tries to force to convert to unicode a text.
    REALLY DIRTY HACK TO TRY TO DETECT ENCODINGS...
    :param text: text to convert
    :return: unicode text
    """
    if type(text) == six.text_type:
        for encoding in ['latin_1', 'ascii', 'utf-8']:
            try:
                strtext = text.encode(encoding)
            except:
                pass
            else:
                break
        text = strtext

    unitext = text
    for encoding in ['utf-8', 'ascii', 'latin_1']:
        try:
            unitext = text.decode(encoding)
        except:
            pass
        else:
            break
    return unitext


def xml_get_tag(xml, tag, parent_tag=None, multi_line=False):
    """
    Returns the tag data for the first instance of the named tag, or for all instances if multi is true.
    If a parent tag is specified, then that will be required before the tag.
    """
    expr_str = '[<:]'+tag+'.*?>(?P<matched_text>.+?)<'
    if parent_tag:
        expr_str = '[<:]'+parent_tag+'.*?>.*?' + expr_str
    expr = re.compile(expr_str, re.DOTALL | re.IGNORECASE)
    if multi_line:
        return expr.findall(xml)
    else:
        if expr.search(xml):
            return expr.search(xml).group('matched_text').strip()
        else:
            return None